/* Discrete Fourier Transform
 * Demonstration code by Emil Mikulic, 2001.
 * Updated July 2003 for libsndfile 1.0.4
 * http://purl.org/net/overload
 *
 * Feel free to do whatever you like with this code.
 * Feel free to credit me.
 */

#include <math.h>
#include <stdio.h>
#include <sndfile.h> /* http://www.zip.com.au/~erikd/libsndfile/ */

#ifdef PI
#undef PI
#endif

/* overkill */
#define PI 3.14159265358979323846264338327950288419716939937510582f

float *data_in;
SF_INFO sfinfo;
SNDFILE *sndfile;
unsigned int samples;



static void dft(void)
{
	int r, k;

	for (r=0; r<=samples/2; r++)
	{
		float re = 0.0f, im = 0.0f;
		float part = (float)r * -2.0f * PI / (float)samples;

		for (k=0; k<samples; k++)
		{
			/*
				float theta = (float)(r*k) * -2.0f * PI /
					(float)samples;
			*/

			float theta = part * (float)k;
			re += data_in[k] * cos(theta);
			im += data_in[k] * sin(theta);
		}

		printf("%f Hz: %f\n",
			(float)r*(float)sfinfo.samplerate/(float)samples,
			sqrt((float)(re*re + im*im)) );
	}
}



int main()
{
	sndfile = sf_open("sample.wav", SFM_READ, &sfinfo);
	if (!sndfile)
	{
		fprintf(stderr, "Can't open file for reading.\n");
		return EXIT_FAILURE;
	}
	samples = (unsigned int)sfinfo.frames;

	fprintf(stderr, "Opened sample --\n\n"
		"samplerate:  %d\n"
		"samples:     %d\n"
		"channels:    %d\n\n",
		sfinfo.samplerate, samples, sfinfo.channels);

	if (sfinfo.channels != 1)
	{
		fprintf(stderr, "Need mono input.\n");
		sf_close(sndfile);
		return EXIT_FAILURE;
	}

	/* Allocate memory */
	data_in = (float*) malloc(sizeof(float) * samples);
	if (!data_in)
	{
		fprintf(stderr, "Out of memory!\n");
		sf_close(sndfile);
		return EXIT_FAILURE;
	}

	/* Fill up input */
	if (sf_read_float(sndfile, data_in, sfinfo.frames) !=
		sfinfo.frames)
	{
		fprintf(stderr, "Failed to read %d samples.\n", samples);
		sf_close(sndfile);
		return EXIT_FAILURE;
	}
	sf_close(sndfile);

	dft();

	free(data_in);
	return EXIT_SUCCESS;
}
